@@ -15,9 +15,9 @@ sub find {
 sub find_one {
   my ($self, $query) = @_;
   $query = {_id => $query} if ref $query eq 'Mango::BSON::ObjectID';
+  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   # Non-blocking
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   return $self->find($query)->limit(-1)->next(
     sub {
       my ($cursor, $err, $doc) = @_;
@@ -34,12 +34,12 @@ sub full_name { join '.', $_[0]->db->name, $_[0]->name }
 sub insert {
   my ($self, $docs) = @_;
   $docs = [$docs] unless ref $docs eq 'ARRAY';
+  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   # Make sure all documents have ids
   my @ids = map { $_->{_id} //= bson_oid } @$docs;
 
   # Non-blocking
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   return $self->db->mango->insert(
     ($self->full_name, {}, @$docs) => sub {
       my ($mango, $err, $reply) = @_;
@@ -55,35 +55,46 @@ sub insert {
 }
 
 sub remove {
-  my $self = shift;
-  my $query = ref $_[0] eq 'CODE' ? {} : shift // {};
-  return $self->_handle('delete', {}, $query, @_);
+  my $self    = shift;
+  my $query   = ref $_[0] eq 'CODE' ? {} : shift // {};
+  my $options = ref $_[0] eq 'CODE' ? {} : shift // {};
+  my $flags   = $options->{single} ? {single_remove => 1} : {};
+  return $self->_handle('delete', $flags, $query, @_);
 }
 
 sub update {
   my ($self, $query, $update) = (shift, shift, shift);
-  return $self->_handle('update', {}, $query, $update, @_);
+  my $options = ref $_[0] eq 'CODE' ? {} : shift // {};
+
+  my $flags = {};
+  $flags->{upsert}       = $options->{upsert};
+  $flags->{multi_update} = $options->{multi};
+
+  return $self->_handle('update', $flags, $query, $update, @_);
 }
 
-sub _error { $_[0]->[5][0]{ok} ? $_[0]->[5][0]{err} : $_[0]->[5][0]{errmsg} }
+sub _error {
+  my $doc = shift->{docs}[0];
+  return $doc->{ok} ? $doc->{err} : $doc->{errmsg};
+}
 
 sub _handle {
   my ($self, $method) = (shift, shift);
+  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   # Non-blocking
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   return $self->db->mango->$method(
     ($self->full_name, @_) => sub {
       my ($mango, $err, $reply) = @_;
       $err ||= _error($reply);
-      $self->$cb($err, $reply->[5][0]);
+      $self->$cb($err, $reply->{docs}[0]);
     }
   ) if $cb;
 
   # Blocking
   my $reply = $self->db->mango->$method($self->full_name, @_);
   if (my $err = _error($reply)) { croak $err }
-  return $reply->[5][0];
+  return $reply->{docs}[0];
 }
 
 1;
@@ -171,29 +182,55 @@ to perform operation non-blocking.
 
   my $doc = $collection->remove;
   my $doc = $collection->remove({foo => 'bar'});
+  my $doc = $collection->remove({foo => 'bar'}, {single => 1});
 
 Remove documents from collection. You can also append a callback to perform
 operation non-blocking.
 
-  $collection->remove({foo => 'bar'} => sub {
+  $collection->remove(({foo => 'bar'}, {single => 1}) => sub {
     my ($collection, $err, $doc) = @_;
     ...
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
+These options are currently available:
+
+=over 2
+
+=item single
+
+Remove only one document.
+
+=back
+
 =head2 update
 
   my $doc = $collection->update({foo => 'bar'}, {foo => 'baz'});
+  my $doc = $collection->update({foo => 'bar'}, {foo => 'baz'}, {multi => 1});
 
 Update document in collection. You can also append a callback to perform
 operation non-blocking.
 
-  $collection->update(({foo => 'bar'}, {foo => 'baz'}) => sub {
+  $collection->update(({foo => 'bar'}, {foo => 'baz'}, {multi => 1}) => sub {
     my ($collection, $err, $doc) = @_;
     ...
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
+These options are currently available:
+
+=over 2
+
+=item multi
+
+Update more than one document.
+
+=item upsert
+
+Insert document if none could be updated.
+
+=back
+
 =head1 SEE ALSO
 
 L<Mango>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
